#ifndef _EFL_UI_LAYOUT_PART_TEXT_EO_H_
#define _EFL_UI_LAYOUT_PART_TEXT_EO_H_

#ifndef _EFL_UI_LAYOUT_PART_TEXT_EO_CLASS_TYPE
#define _EFL_UI_LAYOUT_PART_TEXT_EO_CLASS_TYPE

typedef Eo Efl_Ui_Layout_Part_Text;

#endif

#ifndef _EFL_UI_LAYOUT_PART_TEXT_EO_TYPES
#define _EFL_UI_LAYOUT_PART_TEXT_EO_TYPES


#endif
/** Elementary layout internal part class
 *
 * @since 1.23
 *
 * @ingroup Efl_Ui_Layout_Part_Text
 */
#define EFL_UI_LAYOUT_PART_TEXT_CLASS efl_ui_layout_part_text_class_get()

EAPI EAPI_WEAK const Efl_Class *efl_ui_layout_part_text_class_get(void) EINA_CONST;

#endif
