#ifndef __ECORE_AUDIO_OUT_WASAPI_H__
#define __ECORE_AUDIO_OUT_WASAPI_H__

#include <Eina.h>
#include <Eo.h>

/**
 * @file ecore_audio_obj_out_wasapi.h
 * @brief Ecore_Audio wasapi output.
 */

/**
 * @defgroup ecore_audio_obj_out_wasapi - Ecore_Audio wasapi output
 * @ingroup Ecore_Audio_Group
 * @{
 */
#include "ecore_audio_out_wasapi.eo.h"
/**
 * @}
 */

#endif /* __ECORE_AUDIO_OUT_WASAPI_H__ */
