/**
 * @page authors Authors
 * @author The Rasterman (Carsten Haitzler) <raster@@rasterman.com>
 * @author Gustavo Sverzut Barbieri <barbieri@@profusion.mobi>
 * @author Cedric Bail <cedric.bail@@free.fr>
 * @author Vincent Torri <vtorri@@univ-evry.fr>
 * @author Daniel Kolesa <quaker66@@gmail.com>
 * @author Jaime Thomas <avi.thomas@@gmail.com>
 * @author Swisscom - http://www.swisscom.ch/
 * @author Christopher Michael <devilhorns@@comcast.net>
 * @author Marco Trevisan (Treviño) <mail@@3v1n0.net>
 * @author Michael Bouchaud <michael.bouchaud@@gmail.com>
 * @author Jonathan Atton (Watchwolf) <jonathan.atton@@gmail.com>
 * @author Brian Wang <brian.wang.0721@@gmail.com>
 * @author Mike Blumenkrantz (discomfitor/zmike) <michael.blumenkrantz@@gmail.com>
 * @author Samsung Electronics tbd
 * @author Samsung SAIT tbd
 * @author Brett Nash <nash@@nash.id.au>
 * @author Bruno Dilly <bdilly@@profusion.mobi>
 * @author Rafael Fonseca <rfonseca@@profusion.mobi>
 * @author Hermet Park <hermetpark@@gmail.com>
 * @author Woohyun Jung <wh0705.jung@@samsung.com>
 * @author Jaehwan Kim <jae.hwan.kim@@samsung.com>
 * @author Wonguk Jeong <wonguk.jeong@@samsung.com>
 * @author Leandro A. F. Pereira <leandro@@profusion.mobi>
 * @author Helen Fornazier <helen.fornazier@@profusion.mobi>
 * @author Gustavo Lima Chaves <glima@@profusion.mobi>
 * @author Fabiano Fidêncio <fidencio@@profusion.mobi>
 * @author Tiago Falcão <tiago@@profusion.mobi>
 * @author Otavio Pontes <otavio@@profusion.mobi>
 * @author Viktor Kojouharov <vkojouharov@@gmail.com>
 * @author Daniel Juyung Seo (SeoZ) <juyung.seo@@samsung.com> <seojuyung2@@gmail.com>
 * @author Sangho Park <sangho.g.park@@samsung.com> <gouache95@@gmail.com>
 * @author Rajeev Ranjan (Rajeev) <rajeev.r@@samsung.com> <rajeev.jnnce@@gmail.com>
 * @author Seunggyun Kim <sgyun.kim@@samsung.com> <tmdrbs@@gmail.com>
 * @author Sohyun Kim <anna1014.kim@@samsung.com> <sohyun.anna@@gmail.com>
 * @author Jihoon Kim <jihoon48.kim@@samsung.com>
 * @author Jeonghyun Yun (arosis) <jh0506.yun@@samsung.com>
 * @author Tom Hacohen <tom@@stosb.com>
 * @author Aharon Hillel <a.hillel@@samsung.com>
 * @author Shinwoo Kim <kimcinoo@@gmail.com>
 * @author Govindaraju SM <govi.sm@@samsung.com> <govism@@gmail.com>
 * @author Prince Kumar Dubey <prince.dubey@@samsung.com> <prince.dubey@@gmail.com>
 * @author Sung W. Park <sungwoo@@gmail.com>
 * @author Thierry el Borgi <thierry@@substantiel.fr>
 * @author Shilpa Singh <shilpa.singh@@samsung.com> <shilpasingh.o@@gmail.com>
 * @author Chanwook Jung <joey.jung@@samsung.com> <jchanwook@gmail.com>
 * @author Hyoyoung Chang <hyoyoung.chang@@samsung.com>
 * @author Guillaume "Kuri" Friloux <guillaume.friloux@@asp64.com>
 * @author Kim Yunhan <spbear@@gmail.com>
 * @author Tae-Hwan Kim (Bluezery) <the81.kim@samsung.com> <ohpowel@@gmail.com>
 * @author Nicolas Aguirre <aguirre.nicolas@@gmail.com>
 * @author Sanjeev BA <iamsanjeev@@gmail.com>
 * @author Hyunsil Park <hyunsil.park@@samsung.com>
 * @author Goun Lee <gouni.lee@@samsung.com>
 * @author Mikael Sans <sans.mikael@@gmail.com>
 * @author Doyoun Kang <doyoun.kang@@samsung.com>
 * @author M.V.K. Sumanth <sumanth.m@@samsung.com> <mvksumanth@@gmail.com>
 * @author Jérôme Pinot <ngc891@@gmail.com>
 * @author Davide Andreoli <dave@@gurumeditation.it>
 * @author Michal Pakula vel Rutka <m.pakula@@samsung.com>
 * @author Thiep Ha <thiep.ha@@samsung.com>
 * @author Artem Popov <artem.popov@@samsung.com>
 * @author Minseok Kim <minseok3.kim@@samsung.com>
 * @author Jean-Philippe André <jpeg@@videolan.org>
 * @author JiHyeon Seol <jihyeon.seol@@samsung.com>
 * @author Flavio Ceolin <flavio.ceolin@@profusion.mobi>
 * @author Igor Murzov <e-mail@@date.by>
 * @author Jiyoun Park <jy0703.park@@samsung.com>
 * @author KoziarekBeata <b.koziarek@@samsung.com>
 * @author Daniel Zaoui <daniel.zaoui@@samsung.com>
 * @author Yakov Goldberg <yakov.g@@samsung.com>
 * @author Murilo Belluzzo <murilo.belluzzo@@profusion.mobi>
 * @author Ricardo de Almeida Gonzaga <ricardo@@profusion.mobi>
 * @author Gwanglim Lee <gl77.lee@@samsung.com> <gwanglim@@gmail.com>
 * @author JaeHyun Cho <jae_hyun_cho@@naver.com>
 * @author Bora Hwang <bora1.hwang@@samsung.com>
 * @author Jiyoung Choi <jychoi7.choi@@samsung.com>
 * @author Arvind R <arvino55@@gmail.com>
 * @author Paulo Cavalcanti <paulo.cavalcanti@@linux.intel.com>
 * @author Stefan Schmidt <stefan@@datenfreihafen.org>
 * @author Ryuan Choi (ryuan) <ryuan.choi@@samsung.com> <ryuan.choi@@gmail.com>
 * @author Hosang Kim <hosang12.kim@@samsung.com>
 * @author Youngbok Shin <youngb.shin@@samsung.com>
 * @author Niraj Kumar <niraj.kr@@samsung.com> <niraj.kumar.ait@@gmail.com>
 * @author Amitesh Singh <singh.amitesh@@gmail.com>
 * @author Abhinandan Aryadipta <a.aryadipta@@samsung.com>
 * @author Sanghyeon Lee <sh10233.lee@@samsung.com>
 * @author Anil Kumar Nahak <ak.nahak@@samsung.com>
 * @author Michal Jagiello <m.jagiello@@samsung.com>
 * @author Chinmaya Panigrahi <c.panigrahi@@samsung.com>
 * @author Mohammad Irfan <mohammad.i@@samsung.com>
 * @author ajwillia.ms (Andrew Williams) <andy@@andywilliams.me>
 * @author Iván Briano <sachieru@@gmail.com>
 * @author Jonas M. Gastal <jgastal@@profusion.mobi>
 * @author Rafael Antognolli <antognolli@@gmail.com>
 * @author Mike McCormack <mike@@atratus.org>
 * @author Massimo Maiurana <maiurana@@gmail.com>
 * @author Henrique Dante de Almeida <hdante@@profusion.mobi>
 * @author Lucas De Marchi <lucas.demarchi@@profusion.mobi>
 * @author Boris Faure <billiob@@gmail.com>
 * @author Sebastian Dransfeld <sd@@tango.flipp.net>
 * @author Daniel Willmann <daniel@@totalueberwachung.de>
 * @author maxerba <maiurana@@gmail.com>
 * @author Youness Alaoui <kakaroto@@kakaroto.homelinux.net>
 * @author Rui Seabra <rms@@1407.org>
 * @author Andreas Volz <linux@@brachttal.net>
 * @author Jaeun Choi <jaeun12.choi@@samsung.com>
 * @author Doug Newgard <scimmia22@@outlook.com>
 * @author José Roberto de Souza <zehortigoza@@profusion.mobi>
 * @author Luis Felipe Strano Moraes <lfelipe@@profusion.mobi>
 * @author Thiago Thamada <tiba@@profusion.mobi>
 * @author U. Artie Eoff <ullysses.a.eoff@@intel.com>
 * @author Daniel Hirt <daniel.hirt@@samsung.com>
 * @author Eduardo Lima <eduardo.lima@@intel.com>
 * @author Leif Middelschulte <leif.middelschulte@@gmail.com>
 * @author Lukasz Stanislawski <l.stanislaws@@samsung.com>
 * @author Stephen Houston <smhouston88@@gmail.com>
 * @author Andrii Kroitor <an.kroitor@@samsung.com>
 * @author Joao Paulo Fernandes Ventura <ventura@@profusion.mobi>
 * @author Pau Espin Pedrol <pespin.shar@@gmail.com>
 * @author Jérémy Zurcher <jeremy@@asynk.ch>
 * @author Kai Huuhko <kai.huuhko@@gmail.com>
 * @author Leandro Dorileo <dorileo@@profusion.mobi>
 * @author Michael Jennings <mej@@kainx.org>
 * @author Myungjae Lee <mjae.lee@@samsung.com>
 * @author Tristan Lelong <tristan.lelong@@blunderer.org>
 * @author Yossi Kantor <yossi.kantor@@samsung.com>
 * @author cabelitos <guilherme.iscaro@@intel.com>
 * @author Alex Wu <zhiwen.wu@@linux.intel.com>
 * @author Alex-P. Natsios <drakevr@@linuxteam.teilar.gr>
 * @author Aron Xu <happyaron.xu@@gmail.com>
 * @author Chidambar Zinnoury <illogict@@online.fr>
 * @author Christophe Sadoine <chris@@indefini.org>
 * @author Daniel Vieira Franzolin <daniel@@profusion.mobi>
 * @author Daniele Ricci <daniele.athome@@gmail.com>
 * @author David Walter Seikel <onefang@@gmail.com>
 * @author Dennis Schridde <devurandom@@gmx.net>
 * @author Hannes Janetzek <hannes.janetzek@@gmail.com>
 * @author Lionel Landwerlin <llandwerlin@@gmail.com>
 * @author Mariusz Bialonczyk <manio@@skyboo.net>
 * @author Martin Jansa <Martin.Jansa@@gmail.com>
 * @author Michael Lauer <mlauer@@vanille-media.de>
 * @author Mike Frysinger <vapier@@gentoo.org>
 * @author Rodrigo Cesar Lopes Belem <rclbelem@@gmail.com>
 * @author Seong-ho Cho <darkcircle.0426@@gmail.com>
 * @author Seunghun Lee <shiin.lee@@samsung.com>
 * @author Thanatermesis <thanatermesis.ecvs@@gmail.com>
 * @author Viacheslav Lvov <v.lvov@@samsung.com>
 * @author Deasung Kim <deasung.kim@@samsung.com>
 * @author Jeonghoon Park <jh1979.park@@samsung.com>
 * @author Prashant <pb.ingale@@samsung.com>
 * @author suxia li <suxia.li@@samsung.com>
 * @author yan.wang <yan.wang@@linux.intel.com>
 * @author Anand <anand.km@@samsung.com>
 * @author Subhransu Sekhar Mohanty <sub.mohanty@@samsung.com>
 * @author Rajesh P S <rajeshps@@samsung.com>
 * @author Srivardhan Hebbar <sri.hebbar@@samsung.com>
 * @author Savio Sena <savio@@expertisesolutions.com.br>
 * @author Jae Yong Hwang <j_yong.hwang@@samsung.com>
 * @author Kabeer Khan <kabeer.khan@@samsung.com>
 * @author yinsc <shouchen.yin@@samsung.com>
 * @author Subodh Kumar <s7158.kumar@@samsung.com>
 * @author Kumar Navneet <k.navneet@@samsung.com>
 * @author Godly T Alias <godly.talias@@samsung.com> <godlytalias@@yahoo.co.in>
 * @author Divyesh Purohit <div.purohit@samsung.com> <purohit.div@gmail.com>
 * @author Minkyu Kang <mk7.kang@samsung.com>
 * @author Jinyong Park <j4939.park@samsung.com>
 *
 * Please contact <enlightenment-devel@lists.sourceforge.net> to get in
 * contact with the developers and maintainers.
 */
