/**
 * Add the dayselector.
 *
 * @param parent Parent object
 * @return New dayselector object or @c NULL, if it cannot be created
 *
 * @ingroup Elm_Dayselector
 */
EAPI Evas_Object *elm_dayselector_add(Evas_Object *parent);

#include "elm_dayselector_eo.legacy.h"
