/**
 * Add a new bubble to the parent
 *
 * @param parent The parent object
 * @return The new object or NULL if it cannot be created
 *
 * This function adds a text bubble to the given parent evas object.
 *
 * @ingroup Elm_Bubble
 */
EAPI Evas_Object                 *elm_bubble_add(Evas_Object *parent);

#include "elm_bubble_eo.legacy.h"