/*
 * Autogenerated by the Meson build system.
 * Do not edit, your changes will be lost.
 */

#pragma once

#define EINA_CONFIGURE_HAVE_DIRENT_H 1

#define EINA_ENABLE_LOG 1

#define EINA_HAVE_ALLOCA_H 1

#define EINA_HAVE_BYTESWAP_H 1

#define EINA_HAVE_POSIX_SPINLOCK 1

#define EINA_HAVE_PTHREAD_AFFINITY 1

#define EINA_HAVE_PTHREAD_BARRIER 1

#define EINA_HAVE_PTHREAD_SETNAME 1

#define EINA_HAVE_THREADS 1

#define EINA_MAGIC_DEBUG 1

#define EINA_SAFETY_CHECKS 1

#define EINA_SIZEOF_UINTPTR_T 4

#define EINA_SIZEOF_WCHAR_T 4

