#ifndef ECORE_AUDIO_OUT_SNDFILE_H
#define ECORE_AUDIO_OUT_SNDFILE_H

#include <Eina.h>
#include <Eo.h>

/**
 * @file ecore_audio_obj_out_sndfile.h
 * @brief Ecore_Audio sndfile output.
 */

/**
 * @defgroup ecore_audio_obj_out_sndfile - Ecore_Audio sndfile output
 * @ingroup Ecore_Audio_Group
 * @{
 */
#include "ecore_audio_out_sndfile.eo.h"
/**
 * @}
 */

#endif
