/**
 * @brief Add a separator object to @p parent
 *
 * @param parent The parent object
 *
 * @return The separator object, or NULL upon failure
 *
 * @ingroup Elm_Separator
 */
EAPI Evas_Object *elm_separator_add(Evas_Object *parent);

#include "elm_separator_eo.legacy.h"