/**
 * @addtogroup Elm_Panel_Group
 *
 * @{
 */

/**
 * Panel scroll information
 *
 * @since 1.11
 */
typedef struct _Elm_Panel_Scroll_Info Elm_Panel_Scroll_Info;
/**
 * Panel scroll information
 *
 * @since 1.11
 */
struct _Elm_Panel_Scroll_Info
{
   double rel_x;  /**<content scrolled position (0.0 ~ 1.0) in the panel>*/
   double rel_y;  /**<content scrolled position (0.0 ~ 1.0) in the panel>*/

};

/**
 * @}
 */
