#include "efl_access_object.h"
#include "efl_access_text.h"
#include "efl_access_widget_action.h"
#include "efl_access_window.h"

#ifdef EFL_BETA_API_SUPPORT
#include "efl_access_action.eo.h"
#include "efl_access_component.eo.h"
#include "efl_access_editable_text.eo.h"
#include "efl_access_selection.eo.h"
#include "efl_access_value.eo.h"
#include "efl_ui_legacy.eo.h"
#endif
