/**
 * Add a new plug image to the parent.
 *
 * @param parent The parent object
 * @return The new plug image object or NULL if it cannot be created
 *
 * @ingroup Elm_Plug_Group
 */
EAPI Evas_Object    *elm_plug_add(Evas_Object *parent);

#include "elm_plug_eo.legacy.h"
