#ifndef _ELM_ATSPI_APP_OBJECT_EO_LEGACY_H_
#define _ELM_ATSPI_APP_OBJECT_EO_LEGACY_H_

#ifndef _ELM_ATSPI_APP_OBJECT_EO_CLASS_TYPE
#define _ELM_ATSPI_APP_OBJECT_EO_CLASS_TYPE

typedef Eo Elm_Atspi_App_Object;

#endif

#ifndef _ELM_ATSPI_APP_OBJECT_EO_TYPES
#define _ELM_ATSPI_APP_OBJECT_EO_TYPES


#endif

#endif
