/**
 * Add a new clock widget to the given parent Elementary
 * (container) object
 *
 * @param parent The parent object
 * @return a new clock widget handle or @c NULL, on errors
 *
 * This function inserts a new clock widget on the canvas.
 *
 * @ingroup Elm_Clock
 */
EAPI Evas_Object      *elm_clock_add(Evas_Object *parent);

#include "elm_clock_eo.legacy.h"