EAPI void                   elm_diskselector_item_selected_set(Elm_Object_Item *it, Eina_Bool selected);

EAPI Eina_Bool              elm_diskselector_item_selected_get(const Elm_Object_Item *it);

EAPI Elm_Object_Item *elm_diskselector_item_prev_get(const Elm_Object_Item *it);

EAPI Elm_Object_Item *elm_diskselector_item_next_get(const Elm_Object_Item *it);

