/**
 * @page elementary_main Elementary
 * @image html  elementary.png
 * @version 1.26.3
 * @date 2008-2014
 *
 * @section elm_intro What is Elementary?
 *
 * This is a VERY SIMPLE toolkit. It is not meant for writing extensive desktop
 * applications (yet). Small simple ones with simple needs.
 *
 * It is meant to make the programmers work almost brainlessly but give them lots
 * of flexibility.
 *
 * @li @ref Elm_Start - Go here to quickly get started with writing Apps
 *
 * @section organization Organization
 *
 * One can divide Elementary into three main groups:
 * @li @ref infralist - These are modules that deal with Elementary as a whole.
 * @li @ref widgetslist - These are the widgets you'll compose your UI out of.
 * @li @ref containerslist - These are the containers which hold the widgets.
 *
 * @section license License
 *
 * LGPL v2 (see COPYING in the base of Elementary's source). This applies to
 * all files in the source tree.
 *
 * @section ack Acknowledgements
 * There is a lot that goes into making a widget set, and they don't happen out of
 * nothing. It's like trying to make everyone everywhere happy, regardless of age,
 * gender, race or nationality - and that is really tough. So thanks to people and
 * organizations behind this, as listed in the @ref authors page.
 */
