#ifndef _EFL_UI_SPOTLIGHT_ICON_INDICATOR_EO_H_
#define _EFL_UI_SPOTLIGHT_ICON_INDICATOR_EO_H_

#ifndef _EFL_UI_SPOTLIGHT_ICON_INDICATOR_EO_CLASS_TYPE
#define _EFL_UI_SPOTLIGHT_ICON_INDICATOR_EO_CLASS_TYPE

typedef Eo Efl_Ui_Spotlight_Icon_Indicator;

#endif

#ifndef _EFL_UI_SPOTLIGHT_ICON_INDICATOR_EO_TYPES
#define _EFL_UI_SPOTLIGHT_ICON_INDICATOR_EO_TYPES


#endif
#ifdef EFL_BETA_API_SUPPORT
#define EFL_UI_SPOTLIGHT_ICON_INDICATOR_CLASS efl_ui_spotlight_icon_indicator_class_get()

EAPI EAPI_WEAK const Efl_Class *efl_ui_spotlight_icon_indicator_class_get(void) EINA_CONST;
#endif /* EFL_BETA_API_SUPPORT */

#endif
