#ifndef _EFL_UI_NAVIGATION_BAR_PART_BACK_BUTTON_EO_H_
#define _EFL_UI_NAVIGATION_BAR_PART_BACK_BUTTON_EO_H_

#ifndef _EFL_UI_NAVIGATION_BAR_PART_BACK_BUTTON_EO_CLASS_TYPE
#define _EFL_UI_NAVIGATION_BAR_PART_BACK_BUTTON_EO_CLASS_TYPE

typedef Eo Efl_Ui_Navigation_Bar_Part_Back_Button;

#endif

#ifndef _EFL_UI_NAVIGATION_BAR_PART_BACK_BUTTON_EO_TYPES
#define _EFL_UI_NAVIGATION_BAR_PART_BACK_BUTTON_EO_TYPES


#endif
#ifdef EFL_BETA_API_SUPPORT
/** @ref Efl_Ui_Navigation_Bar internal part back button class.
 *
 * @ingroup Efl_Ui_Navigation_Bar_Part_Back_Button
 */
#define EFL_UI_NAVIGATION_BAR_PART_BACK_BUTTON_CLASS efl_ui_navigation_bar_part_back_button_class_get()

EAPI EAPI_WEAK const Efl_Class *efl_ui_navigation_bar_part_back_button_class_get(void) EINA_CONST;
#endif /* EFL_BETA_API_SUPPORT */

#endif
