/**
 * @addtogroup Elm_Slider
 *
 * @{
 */

typedef char *(*slider_func_type)(double);
typedef char *(*slider_func_full_type)(double, void *);
typedef void (*slider_freefunc_type)(char *);

/**
 * @}
 */
