/**
 * @defgroup Elm_Route Route
 * @ingroup Elementary
 *
 * @image html route_inheritance_tree.png
 * @image latex route_inheritance_tree.eps
 *
 * For displaying a route on the map widget.
 *
 * @{
 */

#ifndef EFL_NOLEGACY_API_SUPPORT
#include "elm_route_legacy.h"
#endif
/**
 * @}
 */
