#ifndef _ELM_CURSOR_H
#define _ELM_CURSOR_H

/**
 * @file
 * @brief Defines the various cursor types for the X Windows system.
 */

#define ELM_CURSOR_X                   "x"
#define ELM_CURSOR_ARROW               "arrow"
#define ELM_CURSOR_BASED_ARROW_DOWN    "based_arrow_down"
#define ELM_CURSOR_BASED_ARROW_UP      "based_arrow_up"
#define ELM_CURSOR_BOAT                "boat"
#define ELM_CURSOR_BOGOSITY            "bogosity"
#define ELM_CURSOR_BOTTOM_LEFT_CORNER  "bottom_left_corner"
#define ELM_CURSOR_BOTTOM_RIGHT_CORNER "bottom_right_corner"
#define ELM_CURSOR_BOTTOM_SIDE         "bottom_side"
#define ELM_CURSOR_BOTTOM_TEE          "bottom_tee"
#define ELM_CURSOR_BOX_SPIRAL          "box_spiral"
#define ELM_CURSOR_CENTER_PTR          "center_ptr"
#define ELM_CURSOR_CIRCLE              "circle"
#define ELM_CURSOR_CLOCK               "clock"
#define ELM_CURSOR_COFFEE_MUG          "coffee_mug"
#define ELM_CURSOR_CROSS               "cross"
#define ELM_CURSOR_CROSS_REVERSE       "cross_reverse"
#define ELM_CURSOR_CROSSHAIR           "crosshair"
#define ELM_CURSOR_DIAMOND_CROSS       "diamond_cross"
#define ELM_CURSOR_DOT                 "dot"
#define ELM_CURSOR_DOT_BOX_MASK        "dot_box_mask"
#define ELM_CURSOR_DOUBLE_ARROW        "double_arrow"
#define ELM_CURSOR_DRAFT_LARGE         "draft_large"
#define ELM_CURSOR_DRAFT_SMALL         "draft_small"
#define ELM_CURSOR_DRAPED_BOX          "draped_box"
#define ELM_CURSOR_EXCHANGE            "exchange"
#define ELM_CURSOR_FLEUR               "fleur"
#define ELM_CURSOR_GOBBLER             "gobbler"
#define ELM_CURSOR_GUMBY               "gumby"
#define ELM_CURSOR_HAND1               "hand1"
#define ELM_CURSOR_HAND2               "hand2"
#define ELM_CURSOR_HEART               "heart"
#define ELM_CURSOR_ICON                "icon"
#define ELM_CURSOR_IRON_CROSS          "iron_cross"
#define ELM_CURSOR_LEFT_PTR            "left_ptr"
#define ELM_CURSOR_LEFT_SIDE           "left_side"
#define ELM_CURSOR_LEFT_TEE            "left_tee"
#define ELM_CURSOR_LEFTBUTTON          "leftbutton"
#define ELM_CURSOR_LL_ANGLE            "ll_angle"
#define ELM_CURSOR_LR_ANGLE            "lr_angle"
#define ELM_CURSOR_MAN                 "man"
#define ELM_CURSOR_MIDDLEBUTTON        "middlebutton"
#define ELM_CURSOR_MOUSE               "mouse"
#define ELM_CURSOR_PENCIL              "pencil"
#define ELM_CURSOR_PIRATE              "pirate"
#define ELM_CURSOR_PLUS                "plus"
#define ELM_CURSOR_QUESTION_ARROW      "question_arrow"
#define ELM_CURSOR_RIGHT_PTR           "right_ptr"
#define ELM_CURSOR_RIGHT_SIDE          "right_side"
#define ELM_CURSOR_RIGHT_TEE           "right_tee"
#define ELM_CURSOR_RIGHTBUTTON         "rightbutton"
#define ELM_CURSOR_RTL_LOGO            "rtl_logo"
#define ELM_CURSOR_SAILBOAT            "sailboat"
#define ELM_CURSOR_SB_DOWN_ARROW       "sb_down_arrow"
#define ELM_CURSOR_SB_H_DOUBLE_ARROW   "sb_h_double_arrow"
#define ELM_CURSOR_SB_LEFT_ARROW       "sb_left_arrow"
#define ELM_CURSOR_SB_RIGHT_ARROW      "sb_right_arrow"
#define ELM_CURSOR_SB_UP_ARROW         "sb_up_arrow"
#define ELM_CURSOR_SB_V_DOUBLE_ARROW   "sb_v_double_arrow"
#define ELM_CURSOR_SHUTTLE             "shuttle"
#define ELM_CURSOR_SIZING              "sizing"
#define ELM_CURSOR_SPIDER              "spider"
#define ELM_CURSOR_SPRAYCAN            "spraycan"
#define ELM_CURSOR_STAR                "star"
#define ELM_CURSOR_TARGET              "target"
#define ELM_CURSOR_TCROSS              "tcross"
#define ELM_CURSOR_TOP_LEFT_ARROW      "top_left_arrow"
#define ELM_CURSOR_TOP_LEFT_CORNER     "top_left_corner"
#define ELM_CURSOR_TOP_RIGHT_CORNER    "top_right_corner"
#define ELM_CURSOR_TOP_SIDE            "top_side"
#define ELM_CURSOR_TOP_TEE             "top_tee"
#define ELM_CURSOR_TREK                "trek"
#define ELM_CURSOR_UL_ANGLE            "ul_angle"
#define ELM_CURSOR_UMBRELLA            "umbrella"
#define ELM_CURSOR_UR_ANGLE            "ur_angle"
#define ELM_CURSOR_WATCH               "watch"
#define ELM_CURSOR_XTERM               "xterm"

#endif // ifndef _ELM_CURSOR_H
