/**
 * @brief Add a new Ctxpopup object to the parent.
 *
 * @param parent Parent object
 * @return New object or @c NULL, if it cannot be created
 *
 * @ingroup Elm_Ctxpopup_Group
 */
EAPI Evas_Object                 *elm_ctxpopup_add(Evas_Object *parent);

#include "elm_ctxpopup_item_eo.legacy.h"
#include "elm_ctxpopup_eo.legacy.h"
