/**
 * @brief Adds a panel object
 *
 * @param parent The parent object
 *
 * @return The panel object, or NULL on failure
 *
 * @ingroup Elm_Panel_Group
 */
EAPI Evas_Object                 *elm_panel_add(Evas_Object *parent);

#include "elm_panel_eo.legacy.h"
