/**
 * Add a new list widget to the given parent Elementary
 * (container) object.
 *
 * @param parent The parent object.
 * @return a new list widget handle or @c NULL, on errors.
 *
 * This function inserts a new list widget on the canvas.
 *
 * @ingroup Elm_List_Group
 */
EAPI Evas_Object                 *elm_list_add(Evas_Object *parent);

#include "elm_list_item_eo.legacy.h"
#include "elm_list_eo.legacy.h"
