#ifndef ECORE_AUDIO_OBJ_IN_H
#define ECORE_AUDIO_OBJ_IN_H

/* ssize_t needed in .eo.h */
#include <sys/types.h>

#include <Eina.h>
#include <Eo.h>

/**
 * @file ecore_audio_obj_in.h
 * @brief Ecore_Audio Input Object.
 */

/**
 * @defgroup ecore_audio_obj_in - Ecore_Audio input object
 * @ingroup Ecore_Audio_Group
 * @{
 */
#include "ecore_audio_in.eo.h"
/**
 * @}
 */

#endif
