#ifndef ELM_WIN_EO_H
# define ELM_WIN_EO_H

/**
 * @ingroup Elm_Win_Group
 *
 * @{
 */

#include "efl_ui_win.eo.h"

EAPI void elm_win_autodel_set(Evas_Object *obj, Eina_Bool autodel);
EAPI Eina_Bool elm_win_autodel_get(const Evas_Object *obj);



/**
 * @}
 */

#endif
