#ifndef ECORE_AUDIO_OBJ_H
#define ECORE_AUDIO_OBJ_H

#include <Eina.h>
#include <Eo.h>

/**
 * @file ecore_audio_obj.h
 * @brief Base Ecore_Audio object.
 */

/**
 * @defgroup ecore_audio_obj - Base Ecore_Audio object
 * @ingroup Ecore_Audio_Group
 * @{
 */
#include "ecore_audio.eo.h"
/**
 * @}
 */

#endif
