#ifndef _EFL_UI_RADIO_GROUP_IMPL_EO_H_
#define _EFL_UI_RADIO_GROUP_IMPL_EO_H_

#ifndef _EFL_UI_RADIO_GROUP_IMPL_EO_CLASS_TYPE
#define _EFL_UI_RADIO_GROUP_IMPL_EO_CLASS_TYPE

typedef Eo Efl_Ui_Radio_Group_Impl;

#endif

#ifndef _EFL_UI_RADIO_GROUP_IMPL_EO_TYPES
#define _EFL_UI_RADIO_GROUP_IMPL_EO_TYPES


#endif
/** Object with the default implementation for @ref Efl_Ui_Radio_Group.
 *
 * @since 1.23
 *
 * @ingroup Efl_Ui_Radio_Group_Impl
 */
#define EFL_UI_RADIO_GROUP_IMPL_CLASS efl_ui_radio_group_impl_class_get()

EAPI EAPI_WEAK const Efl_Class *efl_ui_radio_group_impl_class_get(void) EINA_CONST;

#endif
