typedef Eo Elm_Frame;

/**
 * @brief Add a new frame to the parent
 *
 * @param parent The parent object
 * @return The new object or NULL if it cannot be created
 *
 * @ingroup Elm_Frame_Group
 */
EAPI Evas_Object                 *elm_frame_add(Evas_Object *parent);

#include "efl_ui_frame_eo.legacy.h"
