/**
 * Add a new slideshow widget to the given parent Elementary
 * (container) object
 *
 * @param parent The parent object
 * @return A new slideshow widget handle or @c NULL, on errors
 *
 * This function inserts a new slideshow widget on the canvas.
 *
 * @ingroup Elm_Slideshow
 */
EAPI Evas_Object          *elm_slideshow_add(Evas_Object *parent);

#include "elm_slideshow_item_eo.legacy.h"
#include "elm_slideshow_eo.legacy.h"
