#ifndef _EFL_UI_ALERT_POPUP_PART_TITLE_EO_H_
#define _EFL_UI_ALERT_POPUP_PART_TITLE_EO_H_

#ifndef _EFL_UI_ALERT_POPUP_PART_TITLE_EO_CLASS_TYPE
#define _EFL_UI_ALERT_POPUP_PART_TITLE_EO_CLASS_TYPE

typedef Eo Efl_Ui_Alert_Popup_Part_Title;

#endif

#ifndef _EFL_UI_ALERT_POPUP_PART_TITLE_EO_TYPES
#define _EFL_UI_ALERT_POPUP_PART_TITLE_EO_TYPES


#endif
/** Efl UI Alert Popup internal part class.
 *
 * @since 1.24
 *
 * @ingroup Efl_Ui_Alert_Popup_Part_Title
 */
#define EFL_UI_ALERT_POPUP_PART_TITLE_CLASS efl_ui_alert_popup_part_title_class_get()

EAPI EAPI_WEAK const Efl_Class *efl_ui_alert_popup_part_title_class_get(void) EINA_CONST;

#endif
