/**
 * Add a new flip selector widget to the given parent Elementary
 * (container) widget
 *
 * @param parent The parent object
 * @return a new flip selector widget handle or @c NULL, on errors
 *
 * This function inserts a new flip selector widget on the canvas.
 *
 * @ingroup Elm_Flipselector
 */
EAPI Evas_Object                *elm_flipselector_add(Evas_Object *parent);

#include "elm_flipselector_item_eo.legacy.h"
#include "elm_flipselector_eo.legacy.h"
