/**
 * Add a new actionslider to the parent.
 *
 * @param parent The parent object
 * @return The new actionslider object or NULL if it cannot be created
 *
 * @ingroup Elm_Actionslider
 */
EAPI Evas_Object                *elm_actionslider_add(Evas_Object *parent);

#include "elm_actionslider_eo.legacy.h"
