#ifndef EFL_UI_SPIN_PRIVATE_H
#define EFL_UI_SPIN_PRIVATE_H

#include "Elementary.h"

typedef struct _Efl_Ui_Spin_Data    Efl_Ui_Spin_Data;
struct _Efl_Ui_Spin_Data
{
   double                val, val_min, val_max;
};

#endif
