/**
 * Add a new route object to the parent's canvas
 *
 * @param parent The parent object
 * @return The new object or NULL if it cannot be created
 *
 * @ingroup Elm_Route
 */
EAPI Evas_Object *elm_route_add(Evas_Object *parent);

#include "elm_route_eo.legacy.h"