/**
 * @brief Add a new Hoversel object
 *
 * @param parent The parent object
 * @return The new object or NULL if it cannot be created
 *
 * @ingroup Elm_Hoversel
 */
EAPI Evas_Object                 *elm_hoversel_add(Evas_Object *parent);

#include "elm_hoversel_item_eo.legacy.h"
#include "elm_hoversel_eo.legacy.h"
