#ifndef _EVAS_VG_GRADIENT_EO_LEGACY_H_
#define _EVAS_VG_GRADIENT_EO_LEGACY_H_

typedef Eo Evas_Vg_Gradient;

#endif

#ifndef _EVAS_VG_GRADIENT_LINEAR_EO_LEGACY_H_
#define _EVAS_VG_GRADIENT_LINEAR_EO_LEGACY_H_

typedef Eo Evas_Vg_Gradient_Linear;

#endif

#ifndef _EVAS_VG_GRADIENT_RADIAL_EO_LEGACY_H_
#define _EVAS_VG_GRADIENT_RADIAL_EO_LEGACY_H_

typedef Eo Evas_Vg_Gradient_Radial;

#endif

