#ifndef _ECORE_CON_EET_SERVER_OBJ_EO_LEGACY_H_
#define _ECORE_CON_EET_SERVER_OBJ_EO_LEGACY_H_

#ifndef _ECORE_CON_EET_SERVER_OBJ_EO_CLASS_TYPE
#define _ECORE_CON_EET_SERVER_OBJ_EO_CLASS_TYPE

typedef Eo Ecore_Con_Eet_Server_Obj;

#endif

#ifndef _ECORE_CON_EET_SERVER_OBJ_EO_TYPES
#define _ECORE_CON_EET_SERVER_OBJ_EO_TYPES


#endif

#endif
