/**
 * @brief Add a new colorselector to the parent
 *
 * @param parent The parent object
 * @return The new object or NULL if it cannot be created
 *
 * @ingroup Elm_Colorselector
 */
EAPI Evas_Object *elm_colorselector_add(Evas_Object *parent);

#include "elm_colorselector_eo.legacy.h"
