/**
 * @brief Add a new label to the parent
 *
 * @param parent The parent object
 * @return The new object or NULL if it cannot be created
 *
 * @ingroup Elm_Label_Group
 */
EAPI Evas_Object                *elm_label_add(Evas_Object *parent);

#include "elm_label_eo.legacy.h"
