#ifndef _ELM_VIEW_LIST_EO_LEGACY_H_
#define _ELM_VIEW_LIST_EO_LEGACY_H_

#ifndef _ELM_VIEW_LIST_EO_CLASS_TYPE
#define _ELM_VIEW_LIST_EO_CLASS_TYPE

typedef Eo Elm_View_List;

#endif

#ifndef _ELM_VIEW_LIST_EO_TYPES
#define _ELM_VIEW_LIST_EO_TYPES


#endif






#endif
