#ifndef _ECORE_X_CURSOR_H
#define _ECORE_X_CURSOR_H

/**
 * @file
 * @brief Defines the various cursor types for the X Windows system.
 */

#define ECORE_X_CURSOR_X                   0
#define ECORE_X_CURSOR_ARROW               2
#define ECORE_X_CURSOR_BASED_ARROW_DOWN    4
#define ECORE_X_CURSOR_UP                  6
#define ECORE_X_CURSOR_BOAT                8
#define ECORE_X_CURSOR_BOGOSITY            10
#define ECORE_X_CURSOR_BOTTOM_LEFT_CORNER  12
#define ECORE_X_CURSOR_BOTTOM_RIGHT_CORNER 14
#define ECORE_X_CURSOR_BOTTOM_SIDE         16
#define ECORE_X_CURSOR_BOTTOM_TEE          18
#define ECORE_X_CURSOR_BOX_SPIRAL          20
#define ECORE_X_CURSOR_CENTER_PTR          22
#define ECORE_X_CURSOR_CIRCLE              24
#define ECORE_X_CURSOR_CLOCK               26
#define ECORE_X_CURSOR_COFFEE_MUG          28
#define ECORE_X_CURSOR_CROSS               30
#define ECORE_X_CURSOR_CROSS_REVERSE       32
#define ECORE_X_CURSOR_CROSSHAIR           34
#define ECORE_X_CURSOR_DIAMOND_CROSS       36
#define ECORE_X_CURSOR_DOT                 38
#define ECORE_X_CURSOR_DOT_BOX_MASK        40
#define ECORE_X_CURSOR_DOUBLE_ARROW        42
#define ECORE_X_CURSOR_DRAFT_LARGE         44
#define ECORE_X_CURSOR_DRAFT_SMALL         46
#define ECORE_X_CURSOR_DRAPED_BOX          48
#define ECORE_X_CURSOR_EXCHANGE            50
#define ECORE_X_CURSOR_FLEUR               52
#define ECORE_X_CURSOR_GOBBLER             54
#define ECORE_X_CURSOR_GUMBY               56
#define ECORE_X_CURSOR_HAND1               58
#define ECORE_X_CURSOR_HAND2               60
#define ECORE_X_CURSOR_HEART               62
#define ECORE_X_CURSOR_ICON                64
#define ECORE_X_CURSOR_IRON_CROSS          66
#define ECORE_X_CURSOR_LEFT_PTR            68
#define ECORE_X_CURSOR_LEFT_SIDE           70
#define ECORE_X_CURSOR_LEFT_TEE            72
#define ECORE_X_CURSOR_LEFTBUTTON          74
#define ECORE_X_CURSOR_LL_ANGLE            76
#define ECORE_X_CURSOR_LR_ANGLE            78
#define ECORE_X_CURSOR_MAN                 80
#define ECORE_X_CURSOR_MIDDLEBUTTON        82
#define ECORE_X_CURSOR_MOUSE               84
#define ECORE_X_CURSOR_PENCIL              86
#define ECORE_X_CURSOR_PIRATE              88
#define ECORE_X_CURSOR_PLUS                90
#define ECORE_X_CURSOR_QUESTION_ARROW      92
#define ECORE_X_CURSOR_RIGHT_PTR           94
#define ECORE_X_CURSOR_RIGHT_SIDE          96
#define ECORE_X_CURSOR_RIGHT_TEE           98
#define ECORE_X_CURSOR_RIGHTBUTTON         100
#define ECORE_X_CURSOR_RTL_LOGO            102
#define ECORE_X_CURSOR_SAILBOAT            104
#define ECORE_X_CURSOR_SB_DOWN_ARROW       106
#define ECORE_X_CURSOR_SB_H_DOUBLE_ARROW   108
#define ECORE_X_CURSOR_SB_LEFT_ARROW       110
#define ECORE_X_CURSOR_SB_RIGHT_ARROW      112
#define ECORE_X_CURSOR_SB_UP_ARROW         114
#define ECORE_X_CURSOR_SB_V_DOUBLE_ARROW   116
#define ECORE_X_CURSOR_SHUTTLE             118
#define ECORE_X_CURSOR_SIZING              120
#define ECORE_X_CURSOR_SPIDER              122
#define ECORE_X_CURSOR_SPRAYCAN            124
#define ECORE_X_CURSOR_STAR                126
#define ECORE_X_CURSOR_TARGET              128
#define ECORE_X_CURSOR_TCROSS              130
#define ECORE_X_CURSOR_TOP_LEFT_ARROW      132
#define ECORE_X_CURSOR_TOP_LEFT_CORNER     134
#define ECORE_X_CURSOR_TOP_RIGHT_CORNER    136
#define ECORE_X_CURSOR_TOP_SIDE            138
#define ECORE_X_CURSOR_TOP_TEE             140
#define ECORE_X_CURSOR_TREK                142
#define ECORE_X_CURSOR_UL_ANGLE            144
#define ECORE_X_CURSOR_UMBRELLA            146
#define ECORE_X_CURSOR_UR_ANGLE            148
#define ECORE_X_CURSOR_WATCH               150
#define ECORE_X_CURSOR_XTERM               152

#endif // ifndef _ECORE_X_CURSOR_H
